import json
import logging
import subprocess
import sys
import time
import traceback

import requests

from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.test.testcli_base import TestCase, TestCLI_Base, TestSet
from dtiot_d2c.d2c.utils import color as color
from dtiot_d2c.dmo import ApiClient

log = logging.getLogger(__name__)

# Log levels
DEBUG = 10
INFO = 20
WARNING=30
ERROR = 40
CRITICAL = 50

def print_ERROR(s:str, **kwargs):
    print(f"{color.RED}{s}{color.END}", **kwargs)
    
def print_OK(s:str = "OK", **kwargs):
    print(f"{color.GREEN}{s}{color.END}", **kwargs)

def print_BOLD(s:str, **kwargs):
    print(f"{color.BOLD}{s}{color.END}", **kwargs)       
    
def load_test_configuration(run_id:str, config_file:str, config_var:str)->dict:
    ### Load configuration
    cfg_mod = utils.importModule("test_configuration", config_file)
    cfg = getattr(cfg_mod, config_var)

    # Replace some config variables
    now_secs = str(int(time.time()))
    
    s = json.dumps(cfg)
    for (var_name, var_value) in [["%RUNID%", run_id],
                                  ["%NOWSECS%", now_secs]
                                 ]:
        s = s.replace(var_name, var_value)
    cfg = json.loads(s)
    
    #for (k, v) in cfg.items():
    #    if cfg[k] and type(cfg[k]) == str:
    #        cfg[k] = cfg[k].replace("%RUNID%", run_id)    
            
    return cfg            

def call_has_element_value(d:dict, elem_path:str, test_value):
    b = utils.has_element_value(d, elem_path, test_value)
    #ok = "OK" if b else "NOT OK"
    #print(f"{ok}: has_element_value(node_response, {elem_path}, {test_value})")  
    return b  

def exec_test(label:str, test_func, *args, **kwargs):
    try:
        print(f"{label} ... ", end="", flush=True)
        rv = test_func(*args, **kwargs)
        print_OK(flush=True)
        return rv
    except Exception as ex:
        print_ERROR(f"ERROR: {ex}", flush=True)
        traceback.print_exc()
        raise ex
        #return str(ex)

class TestCase_Sleep(TestCase):
    def __init__(self, api: ApiClient, cfg:dict, run_info:str=""):    
        super().__init__(api=api, cfg=cfg)
        self._run_info = run_info
            
    def get_name(self):
        return self._run_info if self._run_info else "Sleep"
        
    def run(self):
        time.sleep(self.cfg.get("sleep_secs", 5))

    def verify(self):
        pass
        pass

def exec_test_case(test_case_name:str, *args, **kwargs):
    command = ["d2c", test_case_name]

    for arg in args:
        command.append(str(arg))
    for key, val in kwargs.items():
        command.append(key)
        command.append(str(val))
        
    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)

    for line in process.stdout:
        print(line, end='')

    process.wait()
    return process.returncode

def get_message_from_message_monitor(key:str):
    
    url = f"htts://api.scs.iot.telekom.com/message-monitor/body/{key}"
    log.debug(f"GET {url}")
    
    headers = {
        "Content-Type":"application/json",
        "Accept": "application/json"
    }
    log.debug(f"HEADERS: {headers}")
    
    response = requests.get(url, headers=headers)
    
    if not response.ok:
        response.raise_for_status()    

    return response.content    


import json
import logging
import sys
import time
from typing import List

import dtiot_d2c.d2c.test.application as application_test
import dtiot_d2c.d2c.test.device as device_test
import dtiot_d2c.d2c.test.device_group as device_group_test
import dtiot_d2c.d2c.test.utils as tutils
import dtiot_d2c.dmo as dmo
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.device import Device
from dtiot_d2c.d2c.device.consts import *
from dtiot_d2c.d2c.test.testcli_base import TestCase, TestCLI_Base, TestSet
from dtiot_d2c.d2c.utils import color as color
from dtiot_d2c.dmo import ApiClient

log = logging.getLogger(__name__)
log_level = log.getEffectiveLevel() 

DESC = {
    "test-dg01":"test-dg01 - Create device group without anything",
    "test-dg02":"test-dg02 - Create device group with description",
    "test-dg03":"test-dg03 - Modify description of device group",
    "test-dg04":"test-dg04 - Add 1 application to device group",
    "test-dg05-long":"test-dg05-long - Add 1 device to a device group with 1 application - with long names",
    "test-dg05-short":"test-dg05-short - Add 1 device to a device group with 1 application - with short names",
    "test-dg06-short":"test-dg06-short - Add 2 devices to a device group with 1 application - with short names",
    "test-dg07-short":"test-dg07-short - Add 1 devices to a device group with 2 application - with short names",
    "test-dg08-short":"test-dg08-short - Add 2 devices to a device group with 2 application - with short names",
    "test-dg09-short":"test-dg09-short - Add 1 devices to a device group with 0 application - with short names",
    "test-dg10-short":"test-dg10-short - Add 1 application to a device group with 1 device - with short names",
    "test-dg11-short":"test-dg11-short - Add 2 application to a device group with 2 device - with short names",
    "test-dg15-short":"test-dg15-short - Remove 2 devices from device group with 2 applications - with short names",
    "test-dg16-short":"test-dg16-short - Remove 2 applications from device group with 2 devices - with short names",
    "test-dg17-short":"test-dg17-short - Inject an uplink message to a device which is in a device group with 1 applications",
    "test-dg18-short":"test-dg18-short - Inject an uplink message to a device which is in a device group with 2 applications",
    "test-dg20":"test-dg20 - Delete device group",
    "test-dg21":"test-dg21 - Re-create a device group with the same name",
}    

class Test_Base(TestSet):
    def __init__(self, api: ApiClient, test_name:str, test_cfg_name:str):
        super().__init__(api, test_name)
        self._test_cfg_name = test_cfg_name
    def get_sample_configuration_varname(self)->str:
        return self._test_cfg_name

    def test_cleanup(self, cfg:dict):
        device_group_test.delete_device_group(self.api, cfg.get("name"))
        
    def test_pre_cleanup(self, cfg:dict):
        self.test_cleanup(cfg)
        
    def test_post_cleanup(self, cfg:dict):
        self.test_cleanup(cfg)
class Test_Base_CLI(TestCLI_Base):
    def __init__(self, test_class, description:str):
        self._description = description
        self._test_class = test_class 
    def get1LineHelp(self):
        return self._description
    def get_test_set(self, api: ApiClient):
        return self._test_class(api)

###
# MARK: Test Case DG01
###    
class DeviceGroupTest_DG01(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG01", "DeviceGroupCfg_DG01")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
        ]
class DeviceGroupTest_DG01_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG01, DESC.get("test-dg01"))
        
###
# MARK: Test Case DG20
###    
class DeviceGroupTest_DG20(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG20", "DeviceGroupCfg_DG20")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_DeleteDeviceGroup(api=self.api, cfg=cfg),
        ]
class DeviceGroupTest_DG20_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG20, DESC.get("test-dg20"))

###
# MARK: Test Case DG21
###    
class DeviceGroupTest_DG21(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG21", "DeviceGroupCfg_DG21")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_DeleteDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),

        ]
class DeviceGroupTest_DG21_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG21, DESC.get("test-dg21"))
                        
###
# MARK: Test Case DG02
###    
class DeviceGroupTest_DG02(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG02", "DeviceGroupCfg_DG02")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
        ]
class DeviceGroupTest_DG02_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG02, DESC.get("test-dg02"))        
###
# MARK: Test Case DG03
###    
class DeviceGroupTest_DG03(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG03", "DeviceGroupCfg_DG03")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_ModifyDeviceGroup(api=self.api, cfg=cfg),
        ]
class DeviceGroupTest_DG03_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG03, DESC.get("test-dg03"))   
###
# MARK: Test Case DG04
###    
class DeviceGroupTest_DG04(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG04", "DeviceGroupCfg_DG04")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                  device_group_cfg=cfg, 
                                                  application_cfgs=[cfg.get("application_1")]),            
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))

class DeviceGroupTest_DG04_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG04, DESC.get("test-dg04"))                    
###
# MARK: Test Case DG05 long names
###    
class DeviceGroupTest_DG05_long(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG05 long", "DeviceGroupCfg_DG05_long")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]),  
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1")]),                   
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
class DeviceGroupTest_DG05_CLI_long(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG05_long, DESC.get("test-dg05-long"))  
###
# MARK: Test Case DG05 short names
###    
class DeviceGroupTest_DG05_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG05 short", "DeviceGroupCfg_DG05_short")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]),  
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1")]),                   
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
class DeviceGroupTest_DG05_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG05_short, DESC.get("test-dg05-short"))          
###
# MARK: Test Case DG06 short names
###    
class DeviceGroupTest_DG06_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG06 short", "DeviceGroupCfg_DG06_short")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]),  
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1")]),    
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_2")],
                                            verify_application_cfgs=[cfg.get("application_1")]),                   
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_2").get("name"))
class DeviceGroupTest_DG06_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG06_short, DESC.get("test-dg06-short"))          
###
# MARK: Test Case DG07 short names
###    
class DeviceGroupTest_DG07_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG07 short", "DeviceGroupCfg_DG07_short")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]),  
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_2")]),  
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),    
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_2").get("name"))
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
class DeviceGroupTest_DG07_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG07_short, DESC.get("test-dg07-short"))          
###
# MARK: Test Case DG08 short names
###    
class DeviceGroupTest_DG08_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG08 short", "DeviceGroupCfg_DG08_short")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]),  
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_2")]),  
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),    
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_2")],
                                            verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),    
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_2").get("name"))
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_2").get("name"))
class DeviceGroupTest_DG08_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG08_short, DESC.get("test-dg08-short"))          
        
###
# MARK: Test Case DG09 short names
###    
class DeviceGroupTest_DG09_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG09 short", "DeviceGroupCfg_DG09_short")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            # device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            # device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
            #                                                         device_group_cfg=cfg, 
            #                                                         application_cfgs=[cfg.get("application_1")]),  
            # device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            # device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
            #                                                         device_group_cfg=cfg, 
            #                                                         application_cfgs=[cfg.get("application_2")]),  
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    
            # device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            # device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
            #                                 device_group_cfg=cfg, 
            #                                 device_cfgs=[cfg.get("device_2")],
            #                                 verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
            #                                 ),    
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        # application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        # application_test.delete_application(self.api, cfg.get("application_2").get("name"))
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        # device_test.delete_device(self.api, cfg.get("device_2").get("name"))
class DeviceGroupTest_DG09_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG09_short, DESC.get("test-dg09-short"))        
###
# MARK: Test Case DG10 short names
###    
class DeviceGroupTest_DG10_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG10 short", "DeviceGroupCfg_DG10_short") 
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")],
                                                                    verify_device_cfgs=[cfg.get("device_1")]),  

            # device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            # device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
            #                                                         device_group_cfg=cfg, 
            #                                                         application_cfgs=[cfg.get("application_2")]),  
            #                                ),    
            # device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            # device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
            #                                 device_group_cfg=cfg, 
            #                                 device_cfgs=[cfg.get("device_2")],
            #                                 verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
            #                                 ),    
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        # device_test.delete_device(self.api, cfg.get("device_2").get("name"))
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        # application_test.delete_application(self.api, cfg.get("application_2").get("name"))
class DeviceGroupTest_DG10_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG10_short, DESC.get("test-dg10-short"))        

###
# MARK: Test Case DG11 short names
###    
class DeviceGroupTest_DG11_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG11 short", "DeviceGroupCfg_DG11_short") 
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    

            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_2")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    


            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")],
                                                                    verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]),  

            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_2")],
                                                                    verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]),              
        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_2").get("name"))
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_2").get("name"))
class DeviceGroupTest_DG11_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG11_short, DESC.get("test-dg11-short"))        

###
# MARK: Test Case DG15 short names
###    
class DeviceGroupTest_DG15_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG15 short", "DeviceGroupCfg_DG15_short") 
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    

            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_2")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    


            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")],
                                                                    verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]),  

            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_2")],
                                                                    verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]), 
            
            device_group_test.TestCase_RemoveDevicesFromDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    device_cfgs=[cfg.get("device_1")],
                                                                    verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),                         
            device_group_test.TestCase_RemoveDevicesFromDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    device_cfgs=[cfg.get("device_2")],
                                                                    verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),                         

        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_2").get("name"))
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_2").get("name"))
class DeviceGroupTest_DG15_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG15_short, DESC.get("test-dg15-short"))        
###
# MARK: Test Case DG16 short names
###    
class DeviceGroupTest_DG16_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG16 short", "DeviceGroupCfg_DG16_short") 
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    

            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_2")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_2")],
                                            #verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]
                                            ),    


            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")],
                                                                    verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]),  

            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_2")],
                                                                    verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]), 
            
            # device_group_test.TestCase_RemoveDevicesFromDeviceGroup(api=self.api, 
            #                                                         device_group_cfg=cfg, 
            #                                                         device_cfgs=[cfg.get("device_1")],
            #                                                         verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),                         
            # device_group_test.TestCase_RemoveDevicesFromDeviceGroup(api=self.api, 
            #                                                         device_group_cfg=cfg, 
            #                                                         device_cfgs=[cfg.get("device_2")],
            #                                                         verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),       
            
            device_group_test.TestCase_RemoveApplicationsFromDeviceGroup(api=self.api, 
                                                                         device_group_cfg=cfg, 
                                                                         application_cfgs=[cfg.get("application_1")],
                                                                         verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]),                              
            device_group_test.TestCase_RemoveApplicationsFromDeviceGroup(api=self.api, 
                                                                         device_group_cfg=cfg, 
                                                                         application_cfgs=[cfg.get("application_2")],
                                                                         verify_device_cfgs=[cfg.get("device_1"), cfg.get("device_2")]),                              

        ]
    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        device_test.delete_device(self.api, cfg.get("device_2").get("name"))
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_2").get("name"))
class DeviceGroupTest_DG16_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG16_short, DESC.get("test-dg16-short"))        
                                
###
# MARK: Test Case DG17 short names
###    
class DeviceGroupTest_DG17_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG17 short", "DeviceGroupCfg_DG17_short") 
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]), 
             
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1")]),    

            tutils.TestCase_Sleep(api=self.api, cfg=cfg, run_info="Sleep to let d2c service bring uplink message forwarding in placed"),
            
            device_group_test.TestCase_InjectUplinkMessage(api=self.api,
                                                           device_group_cfg=cfg,
                                                           device_cfg=cfg.get("device_1"),
                                                           verify_application_cfgs=[cfg.get("application_1")],
                                                           ),
        ]

    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
class DeviceGroupTest_DG17_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG17_short, DESC.get("test-dg17-short"))        

###
# MARK: Test Case DG18 short names
###    
class DeviceGroupTest_DG18_short(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test DG18 short", "DeviceGroupCfg_DG18_short") 
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_group_test.TestCase_CreateDeviceGroup(api=self.api, cfg=cfg),
            
            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_1")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_1")]), 

            device_group_test.TestCase_CreateApplication(api=self.api, cfg=cfg.get("application_2")),
            device_group_test.TestCase_AddApplicationsToDeviceGroup(api=self.api, 
                                                                    device_group_cfg=cfg, 
                                                                    application_cfgs=[cfg.get("application_2")]), 
             
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg.get("device_1")),      
            device_group_test.TestCase_AddDevicesToDeviceGroup(api=self.api, 
                                            device_group_cfg=cfg, 
                                            device_cfgs=[cfg.get("device_1")],
                                            verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")]),    

            tutils.TestCase_Sleep(api=self.api, cfg=cfg, run_info="Sleep to let d2c service bring uplink message forwarding in placed"),
            
            device_group_test.TestCase_InjectUplinkMessage(api=self.api,
                                                           device_group_cfg=cfg,
                                                           device_cfg=cfg.get("device_1"),
                                                           verify_application_cfgs=[cfg.get("application_1"), cfg.get("application_2")],
                                                           ),
        ]

    def test_cleanup(self, cfg:dict)->str:
        device_group_test.delete_device_group(self.api, cfg.get("name"))   
        device_test.delete_device(self.api, cfg.get("device_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_1").get("name"))
        application_test.delete_application(self.api, cfg.get("application_2").get("name"))
class DeviceGroupTest_DG18_CLI_short(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceGroupTest_DG18_short, DESC.get("test-dg18-short"))        
                                                                


import json
import logging
import sys
import time
from typing import List

import dtiot_d2c.d2c.test.device as device_test
import dtiot_d2c.d2c.test.utils as tutils
import dtiot_d2c.dmo as dmo
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.device import Device
from dtiot_d2c.d2c.device.consts import *
from dtiot_d2c.d2c.test.testcli_base import TestCase, TestCLI_Base, TestSet
from dtiot_d2c.d2c.utils import color as color
from dtiot_d2c.dmo import ApiClient

log = logging.getLogger(__name__)
log_level = log.getEffectiveLevel() 

DESC = {
    "test-d01":"test-d01: create device without anything",
    "test-d02":"test-d02: create device with psk credentials",
    "test-d03":"test-d03: create device with various dvi attributes",
    "test-d04":"test-d04: create device with uplink properties",
    "test-d05":"test-d05: create device with device properties",
    "test-d06":"test-d06: Update device dvi attributes",
    "test-d07":"test-d07: Update device device by adding device properties",
    "test-d08":"test-d08: Update device device by updating device properties",
    "test-d09":"test-d09: Update device by deleting device properties",
    "test-d10":"test-d10: Update device by adding uplink properties",
    "test-d11":"test-d11: Update device by updating uplink properties",
    "test-d13":"test-d13: Delete device",
    "test-d14":"test-d14: Re-create device with same name",
    "test-d15":"test-d15: Inject JSON uplink message",
}    

class Test_Base(TestSet):
    def __init__(self, api: ApiClient, test_name:str, test_cfg_name:str):
        super().__init__(api, test_name)
        self._test_cfg_name = test_cfg_name
    def get_sample_configuration_varname(self)->str:
        return self._test_cfg_name

    def test_cleanup(self, cfg:dict)->str:
        name = cfg.get("name")
        def operation():
            if (device := Device.get(self.api, name=name)):
                Device.delete(self.api, name=name)
            return None
        return tutils.exec_test(f"Deleting device {name}", operation)

    def test_pre_cleanup(self, cfg:dict):
        self.test_cleanup(cfg)

    def test_post_cleanup(self, cfg:dict)->str:
        self.test_cleanup(cfg)
class Test_Base_CLI(TestCLI_Base):
    def __init__(self, test_class, description:str):
        self._description = description
        self._test_class = test_class 
    def get1LineHelp(self):
        return self._description
    def get_test_set(self, api: ApiClient):
        return self._test_class(api)
        
###
# MARK: Test Case D01
###    
class DeviceTest_D01(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D01", "DeviceCfg_D01")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
        ]
class DeviceTest_D01_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D01, DESC.get("test-d01"))
###
# MARK: Test Case D02
###    
class DeviceTest_D02(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D02", "DeviceCfg_D02")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
        ]
class DeviceTest_D02_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D02, DESC.get("test-d02"))
###
# MARK: Test Case D03
###    
class DeviceTest_D03(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D03", "DeviceCfg_D03")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
        ]
class DeviceTest_D03_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D03, DESC.get("test-d03"))
###
# MARK: Test Case D04
###    
class DeviceTest_D04(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D04", "DeviceCfg_D04")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
        ]
class DeviceTest_D04_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D04, DESC.get("test-d04"))
###
# MARK: Test Case D05
###    
class DeviceTest_D05(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D05", "DeviceCfg_D05")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
        ]
class DeviceTest_D05_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D05, DESC.get("test-d05"))
###
# MARK: Test Case D06
###    
class DeviceTest_D06(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D06", "DeviceCfg_D06")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_ModifyDevice(api=self.api, cfg=cfg),            
        ]
class DeviceTest_D06_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D06, DESC.get("test-d06"))
###
# MARK: Test Case D07
###    
class DeviceTest_D07(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D07", "DeviceCfg_D07")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_ModifyDevice(api=self.api, cfg=cfg),                  
        ]
class DeviceTest_D07_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D07, DESC.get("test-d07"))
###
# MARK: Test Case D08
###    
class DeviceTest_D08(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D08", "DeviceCfg_D08")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_ModifyDevice(api=self.api, cfg=cfg),                  
        ]
class DeviceTest_D08_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D08, DESC.get("test-d08"))
###
# MARK: Test Case D09
###    
class DeviceTest_D09(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D09", "DeviceCfg_D09")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_ModifyDevice(api=self.api, cfg=cfg),                  
        ]
class DeviceTest_D09_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D09, DESC.get("test-d09"))
###
# MARK: Test Case D10
###    
class DeviceTest_D10(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D10", "DeviceCfg_D10")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_ModifyDevice(api=self.api, cfg=cfg),                  
        ]
class DeviceTest_D10_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D10, DESC.get("test-d10"))
###
# MARK: Test Case D11
###    
class DeviceTest_D11(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D11", "DeviceCfg_D11")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_ModifyDevice(api=self.api, cfg=cfg),                  
        ]
class DeviceTest_D11_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D11, DESC.get("test-d11"))
###
# MARK: Test Case D13
###    
class DeviceTest_D13(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D13", "DeviceCfg_D13")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            tutils.TestCase_Sleep(api=self.api, cfg=cfg, run_info="Sleep to let d2c service create device in IMPACT"),
            device_test.TestCase_DeleteDevice(api=self.api, cfg=cfg)
        ]
class DeviceTest_D13_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D13, DESC.get("test-d13"))
###
# MARK: Test Case D14
###    
class DeviceTest_D14(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D14", "DeviceCfg_D14")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            tutils.TestCase_Sleep(api=self.api, cfg=cfg, run_info="Sleep to let d2c service create device in IMPACT"),
            device_test.TestCase_DeleteDevice(api=self.api, cfg=cfg),
            tutils.TestCase_Sleep(api=self.api, cfg=cfg, run_info="Sleep to let d2c service delete the device in IMPACT"),
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            tutils.TestCase_Sleep(api=self.api, cfg=cfg, run_info="Sleep to let d2c service create device in IMPACT"),
            device_test.TestCase_DeleteDevice(api=self.api, cfg=cfg)
        ]
class DeviceTest_D14_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D14, DESC.get("test-d14"))
       
       
###
# MARK: Test Case D15
###    
class DeviceTest_D15(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test D15", "DeviceCfg_D15")
    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            device_test.TestCase_CreateDevice(api=self.api, cfg=cfg),
            device_test.TestCase_InjectUplinkMessage(api=self.api, cfg=cfg),
        ]
class DeviceTest_D15_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(DeviceTest_D15, DESC.get("test-d15"))       


import json
import logging

import dtiot_d2c.d2c.test.utils as tutils
import dtiot_d2c.dmo as dmo
from dtiot_d2c.cli.cli_command import ApiConnCmdLineInterface
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.application import Application
from dtiot_d2c.d2c.application.consts import *
from dtiot_d2c.d2c.device.consts import *
from dtiot_d2c.d2c.test.test_cases_application import DESC as APPL_DESC
from dtiot_d2c.d2c.test.test_cases_device import DESC as DEV_DESC
from dtiot_d2c.d2c.test.test_cases_device_group import DESC as DEVGRP_DESC
from dtiot_d2c.d2c.utils import color as color

log = logging.getLogger(__name__)
log_level = log.getEffectiveLevel() 

class AllTestCasesRunner_CLI(ApiConnCmdLineInterface):
    def get1LineHelp(self):
        return "Runs all test cases of a specific test object."

    def addCmdLineArgs(self, argsParser):

        argsParser.add_argument("-to", "--test-object", metavar="<test-object>", 
                                dest="test_object", required=True,
                                choices= ["application", "device", "devicegroup", "all"],
                                help="Test object for which all test cases are run. Possible values are application, device, devicegroup, all")

        argsParser.add_argument("-i", "--test-run-id", metavar="<string>", 
                                dest="run_id", required=True,
                                help="Id of the test run.")

        argsParser.add_argument("-cf", "--test-configuration-file", metavar="<file>", 
                                dest="config_file", required=False,
                                help="Python file which contains the configuration of the test.")

        argsParser.add_argument("-cd", "--test-configuration-dict", metavar="<variable-name>", 
                                dest="config_varname", required=False,
                                help="Name of the dictionary variable within the configuration file with the configuration.")

        argsParser.add_argument("-spr", "--skip-pre-cleanup", dest="skip_pre_cleanup", action="store_const", 
                                const=True, default=False,
                                help="If set, the pre cleanup function is skipped.")

        argsParser.add_argument("-spo", "--skip-post-cleanup", dest="skip_post_cleanup", action="store_const", 
                                const=True, default=False,
                                help="If set, the post cleanup function is skipped.")

        argsParser.add_argument("-n", "--number-of-test-runs", metavar="<integer>", 
                                dest="n", type=int, default=1,
                                help="Number of test runs. Default is 1.")

    def main(self, cmdargs=None):
        
        if cmdargs.test_object == "application":
            DESC = APPL_DESC
        elif cmdargs.test_object == "device":
            DESC = DEV_DESC
        elif cmdargs.test_object == "devicegroup":
            DESC = DEVGRP_DESC
        elif cmdargs.test_object == "all":
            DESC = {}
            DESC.update(APPL_DESC)
            DESC.update(DEV_DESC)
            DESC.update(DEVGRP_DESC)
        
        # Build args
        args = []
        if cmdargs.skip_pre_cleanup:
            args.append("--skip-pre-cleanup")            
        if cmdargs.skip_post_cleanup:
            args.append("--skip-post-cleanup")            

        # Build kwargs
        kwargs = {}
        if cmdargs.config_file:
            kwargs["--test-configuration-file"] = cmdargs.config_file            
        if cmdargs.config_varname:
            kwargs["--test-configuration-dict"] = cmdargs.config_varname            
        if cmdargs.n:
            kwargs["--number-of-test-runs"] = cmdargs.n            

        kwargs["--test-run-id"] = cmdargs.run_id

        ok_tests = []
        error_tests = []

        for test_name, test_desc in DESC.items():
            kwargs["--test-run-id"] = int(kwargs["--test-run-id"]) + 1
            exit_code = tutils.exec_test_case(test_name, *args, **kwargs)
            if exit_code == 0:
                ok_tests.append(test_name)
            else:
                error_tests.append(test_name)

        print("")        
        print(f"{'='*80}")
        print(f"Ok Tests:")
        for t in ok_tests:
            print(f"  {DESC[t]}")

        print("")

        print(f"Error Tests:")
        for t in error_tests:
            print(f"  {DESC[t]}")
        
                       





import json
import logging
import os
import subprocess
import sys
import time
from typing import List

import dtiot_d2c.d2c.test.application as application_test
import dtiot_d2c.d2c.test.utils as tutils
import dtiot_d2c.dmo as dmo
from dtiot_d2c.cli.cli_command import ApiConnCmdLineInterface
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.application import Application
from dtiot_d2c.d2c.application.consts import *
from dtiot_d2c.d2c.device.consts import *
from dtiot_d2c.d2c.get_response import GetResponse
from dtiot_d2c.d2c.test.testcli_base import TestCase, TestCLI_Base, TestSet
from dtiot_d2c.d2c.utils import color as color
from dtiot_d2c.dmo import ApiClient

log = logging.getLogger(__name__)
log_level = log.getEffectiveLevel() 

DESC = {
    "test-a01":"test-a01 - Create application without anything",
    "test-a02":"test-a02 - Create application with single url",
    "test-a03":"test-a03 - Create application with multiple urls",
    "test-a04":"test-a04 - Create application with urls and connection properties",
    "test-a04-short-header": "test-a04-short-header - Create application with urls and short connection properties",
    "test-a05": "test-a05 - Update current url",
    "test-a06": "test-a06 - Add url",
    "test-a07": "test-a07 - Delete url",
    "test-a08": "test-a08 - Add connection property",
    "test-a08-short-header": "test-a08-short-header - Add connection property with short header",
    "test-a09": "test-a09 - Modify connection property",
    "test-a09-short-header": "test-a09-short-header - Modify connection property with short header",
    "test-a10": "test-a10 - Delete connection property",
    "test-a11": "test-a11 - Delete application",
    "test-a12": "test-a12 - Re-create application with the same name",
}
        
class Test_Base(TestSet):
    def __init__(self, api: ApiClient, test_name:str, test_cfg_name:str):
        super().__init__(api, test_name)
        self._test_cfg_name = test_cfg_name
    def get_sample_configuration_varname(self)->str:
        return self._test_cfg_name
    def test_pre_cleanup(self, cfg:dict)->str:
        application_test.delete_application(self.api, cfg.get("name"))
    def test_post_cleanup(self, cfg:dict)->str:
        application_test.delete_application(self.api, cfg.get("name"))

class Test_Base_CLI(TestCLI_Base):
    def __init__(self, test_class, description:str):
        self._description = description
        self._test_class = test_class 
    def get1LineHelp(self):
        return self._description
    def get_test_set(self, api: ApiClient):
        return self._test_class(api)
        
###
# MARK: Test Case A01
###    
class ApplicationTest_A01(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A01", "ApplicationCfg_A01")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A01_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A01, DESC.get("test-a01"))
###
# MARK: Test Case A02
###    
class ApplicationTest_A02(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A02", "ApplicationCfg_A02")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A02_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A02, DESC.get("test-a02"))
###
# MARK: Test Case A03
###    
class ApplicationTest_A03(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A03", "ApplicationCfg_A03")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A03_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A03, DESC.get("test-a03"))
###
# MARK: Test Case A04
###    
class ApplicationTest_A04(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A04", "ApplicationCfg_A04")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A04_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A04, DESC.get("test-a04"))
###
# MARK: Test Case A04 short header
###    
class ApplicationTest_A04_short_header(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A04 short header", "ApplicationCfg_A04_short_header")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A04_CLI_short_header(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A04_short_header, DESC.get("test-a04-short-header"))
###
# MARK: Test Case A05
###    
class ApplicationTest_A05(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A05", "ApplicationCfg_A05")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A05_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A05, DESC.get("test-a05"))
###
# MARK: Test Case A06
###    
class ApplicationTest_A06(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A06", "ApplicationCfg_A06")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A06_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A06, DESC.get("test-a06"))
###
# MARK: Test Case A07
###    
class ApplicationTest_A07(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A07", "ApplicationCfg_A07")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A07_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A07, DESC.get("test-a07"))
###
# MARK: Test Case A08
###    
class ApplicationTest_A08(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A08", "ApplicationCfg_A08")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A08_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A08, DESC.get("test-a08"))
###
# MARK: Test Case A08 short header
###    
class ApplicationTest_A08_short_header(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A08 short header", "ApplicationCfg_A08_short_header")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A08_CLI_short_header(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A08_short_header, DESC.get("test-a04-short-header"))
###
# MARK: Test Case A09
###    
class ApplicationTest_A09(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A09", "ApplicationCfg_A09")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A09_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A09, DESC.get("test-a09"))
###
# MARK: Test Case A09 short header
###    
class ApplicationTest_A09_short_header(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A09 short header", "ApplicationCfg_A09_short_header")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A09_CLI_short_header(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A09_short_header, DESC.get("test-a09-short-header"))
###
# MARK: Test Case A10
###    
class ApplicationTest_A10(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A10", "ApplicationCfg_A10")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A10_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A10, DESC.get("test-a10"))
###
# MARK: Test Case A11
###    
class ApplicationTest_A11(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A11", "ApplicationCfg_A11")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_DeleteApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A11_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A11, DESC.get("test-a11"))
###
# MARK: Test Case A12
###    
class ApplicationTest_A12(Test_Base):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Test A12", "ApplicationCfg_A12")

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
            application_test.TestCase_DeleteApplication(api=self.api, cfg=cfg),
            application_test.TestCase_CreateApplication(api=self.api, cfg=cfg),
        ]
class ApplicationTest_A12_CLI(Test_Base_CLI):
    def __init__(self):
        super().__init__(ApplicationTest_A12, DESC.get("test-a12"))
        



Device01Cfg = {
    "name":"device01-%RUNID%",
    "create_props" : {
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {"address":"Kuckhoffstr 114A, 13156 Berlin"},
        "device_properties": {"lastMaintenance":"25.3.2025 1300", "prop2":"value2"},
        "credentials_id": "device01-%RUNID%",
        "credentials_secret": "Q3hhc21FTUw3NFNiUlIzUw=="
    },
    "modify_props" : {
        "manufacturer":"x PSsystec",
        "model":"x Smartbox Mini",
        "sub_model":"x NB-IoT",
        "iccid": "99999999317959919924",
        "hardware_version":"x 2024.1 PC r1",
        "country":"XX",
        "description":"x Das ist eine Beschreibung.",
        "label":"x this is a label" ,
        "device_type":"x Pretty Device" ,
        "device_name":"Hallo die Waldfee device_name",
        "firmware_version":"x fw 1.0.1",
        "software_version":"x sw 2.0.1",
        "os_version":"x os 3.0.1",
        "location":"geo:88.999999,88.999999" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {"address":"xx Kuckhoffstr 114A, 13156 Berlin", "location":"Berlin"},
        "device_properties": {"lastMaintenance":"25.3.2025 1300","Status": "Out of order"},

        #Update lwm2m cridentials is not supported yet via d2ccli
        #"credentials_id": "x device01-%RUNID%",
        #"credentials_secret": "x Q3hhc21FTUw3NFNiUlIzUw=="
    }    
} 
Device02Cfg = {
    "name":"device02-%RUNID%",
    "create_props" : {
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {"address":"Kuckhoffstr 114A, 13156 Berlin"},
        "device_properties": {"lastMaintenance":"25.3.2025 1300"},
        "credentials_id": "device01-%RUNID%",
        "credentials_secret": "Q3hhc21FTUw3NFNiUlIzUw=="
    }
} 

Application01Cfg = {
    "name":"application01-%RUNID%",
    "create_props" : {
        "application_type":"webHook",
        "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        "connection_properties": {"Auth":"dfadfdsafasdf", "another-token":"88888888"}
    }, 
    "modify_props" : {    
        "urls": ["https://dev.scs.iot.telekom.com/scs-callback-dummy"],
        "connection_properties": {"Auth":"88888888", "another-token":"dfadfdsafasdf"}
    }
}

DeviceGroup01Cfg = {
    "name":"devicegroup01-%RUNID%", 
    "create_props" : {
        "labels":{"deviceType":"SDI People Counter"},
        "device_ids": [],
        "application_ids": [],
        "description": "This is a description"
    },
    "modify_props":{
        "labels":{"deviceType":"SDI People Counter, ELSYS", "street":"Kuckhoffstr."},
        "device_ids": [],
        "application_ids": [],
        "description": "This is an updated description"
    }
}

DeviceGroup02Cfg = {
    "name":"devicegroup02-%RUNID%", 
    "create_props" : {
        "labels":{"deviceType":"SDI People Counter"},
        "device_ids": [],
        "application_ids": [],
        "description": "This is a description"        
    },
    "application_1": {
        "name":"devicegroup02-app1-%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
            "labels": {"customerKey":"CUST-001"},        
        }
    },
    "application_2":{
        "name":"devicegroup02-app2-%RUNID%",
        "create_props":{
            "urls": ["https://dev.scs.iot.telekom.com/scs-callback-dummy"],
            "labels": {"customerKey":"CUST-099", "costCenter":"999888"},        
        }
    },
    "device_1": Device01Cfg,
    "device_2": Device02Cfg
}
###################################################################
###################################################################
# MARK: Application test configurations
###################################################################
###################################################################
ApplicationCfg_A01 = {
    "name":"application-a01-%RUNID%",
    "create_props" : {},
    "modify_props" : {}
}
ApplicationCfg_A02 = {
    "name":"application-a02-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor"
        ],
    }, 
    "modify_props" : {}
}
ApplicationCfg_A03 = {
    "name":"application-a03-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
    }, 
    "modify_props" : {}
}
ApplicationCfg_A04 = {
    "name":"application-a04-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYW5kLmJhbGRpbkB0ZWxla29tLmRlOm42NVIkRXh6bkVwTVltTWY=", 
            "log-level":"debug"
        }        
    }, 
    "modify_props" : {}
}
ApplicationCfg_A04_short_header = {
    "name":"application-a04-short-header-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYW5kLVltTWY=", 
            "log-level":"debug"
        }        
    }, 
    "modify_props" : {}
}
ApplicationCfg_A05 = {
    "name":"application-a05-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor"
        ],
    }, 
    "modify_props" : {
        "urls": [
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],        
    }
}
ApplicationCfg_A06 = {
    "name":"application-a06-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor"
        ],
    }, 
    "modify_props" : {
        "urls": [
            "+https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],   
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ]
    }
}
ApplicationCfg_A07 = {
    "name":"application-a07-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
    }, 
    "modify_props" : {
        "urls": [
            "-https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ], 
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
        ]               
    }
}
ApplicationCfg_A08 = {
    "name":"application-a08-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYW5kLmJhbGRpbkB0ZWxla29tLmRlOm42NVIkRXh6bkVwTVltTWY=", 
            "log-level":"debug"
        }        
    }, 
    "modify_props" : {
        "connection_properties": {
            "+another-prop":"Hallo"
        },
        "connection_properties_verify":{
            "Authorization":"cm9sYW5kLmJhbGRpbkB0ZWxla29tLmRlOm42NVIkRXh6bkVwTVltTWY=", 
            "log-level":"debug",
            "another-prop":"Hallo"                        
        },
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],        
    }
}
ApplicationCfg_A08_short_header = {
    "name":"application-a08-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYTWY=", 
            "log-level":"debug"
        }        
    }, 
    "modify_props" : {
        "connection_properties": {
            "+another-prop":"Hallo"
        },
        "connection_properties_verify":{
            "Authorization":"cm9sYTWY=", 
            "log-level":"debug",
            "another-prop":"Hallo"                        
        },
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],        
    }
}
ApplicationCfg_A09 = {
    "name":"application-a09-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYW5kLmJhbGRpbkB0ZWxla29tLmRlOm42NVIkRXh6bkVwTVltTWY=",             
            "log-level":"debug",
            "another-prop":"Hallo"
        }        
    }, 
    "modify_props" : {
        "connection_properties": {
            "+another-prop":"Hihihihih"
        },
        "connection_properties_verify":{
            "Authorization":"cm9sYW5kLmJhbGRpbkB0ZWxla29tLmRlOm42NVIkRXh6bkVwTVltTWY=",             
            "log-level":"debug",
            "another-prop":"Hihihihih"                        
        },
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],                       
    }
}
ApplicationCfg_A09_short_header = {
    "name":"application-a09-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYTWY=", 
            "log-level":"debug",
            "another-prop":"Hallo"
        }        
    }, 
    "modify_props" : {
        "connection_properties": {
            "+another-prop":"Hihihihih"
        },
        "connection_properties_verify":{
            "Authorization":"cm9sYTWY=", 
            "log-level":"debug",
            "another-prop":"Hihihihih"                        
        },
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],                       
    }
}
ApplicationCfg_A10 = {
    "name":"application-a10-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "connection_properties": {
            "Authorization":"cm9sYTWY=", 
            #"Authorization":"cm9sYW5kLmJhbGRpbkB0ZWxla29tLmRlOm42NVIkRXh6bkVwTVltTWY=",                    
            "log-level":"debug",
            "another-prop":"Hallo"
        }        
    }, 
    "modify_props" : {
        "connection_properties": {
            "-another-prop":None
        },
        "connection_properties_verify":{
            "Authorization":"cm9sYTWY=", 
            "log-level":"debug",
        },
        "urls_verify": [
            "https://api.scs.iot.telekom.com/message-monitor",
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],                                  
    }
}
ApplicationCfg_A11 = {
    "name":"application-a11-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
        ],
        "connection_properties": {}
    }, 
    "modify_props" : {}
}
ApplicationCfg_A12 = {
    "name":"application-a12-%RUNID%",
    "create_props" : {
        "urls": [
            "https://api.scs.iot.telekom.com/message-monitor",
        ],
        "connection_properties": {}
    }, 
    "modify_props" : {}
}

###################################################################
###################################################################
# MARK: Device test configurations
###################################################################
###################################################################
DeviceCfg_D01 = {
    "name":"device-d01-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M"
    },
    "modify_props" : {}
}
DeviceCfg_D02 = {
    "name":"device-d02-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "credentials_id": "device-d02-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA=="        
    },
    "modify_props" : {}
}
DeviceCfg_D03 = {
    "name":"devic-d03-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {},
        "credentials_id": "device-d03-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {}
} 
DeviceCfg_D04 = {
    "name":"devic-d04-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {"address":"Kuckhoffstr 114A, 13156 Berlin"},
        "device_properties": {},
        "credentials_id": "device-d04-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {}
} 
DeviceCfg_D05 = {
    "name":"devic-d05-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {"lastMaintenance":"25.3.2025 1300", "prop2":"value2"},        
        "credentials_id": "device-d05-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {}
} 
DeviceCfg_D06 = {
    "name":"device-d06-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {},
        "credentials_id": "device-d06-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {
        "manufacturer":"x PSsystec",
        "model":"x Smartbox Mini",
        "sub_model":"x NB-IoT",
        "iccid": "99999999317959919924",
        "hardware_version":"x 2024.1 PC r1",
        "country":"XX",
        "description":"x Das ist eine Beschreibung.",
        "label":"x this is a label" ,
        "device_type":"x Pretty Device" ,
        "device_name":"Hallo die Waldfee device_name",
        "firmware_version":"x fw 1.0.1",
        "software_version":"x sw 2.0.1",
        "os_version":"x os 3.0.1",
        "location":"geo:88.999999,88.999999" ,
        #Update lwm2m cridentials is not supported yet via d2ccli
        #"credentials_id": "x device01-%RUNID%",
        #"credentials_secret": "x Q3hhc21FTUw3NFNiUlIzUw=="
    }    
} 
DeviceCfg_D07 = {
    "name":"device-d07-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {
            "lastMaintenance":"25.3.2025 1300", 
            "prop2":"value2"
        },        
        "credentials_id": "device-d07-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {
        "device_properties": {
            "+prop77":"value77", 
            "+prop88":"value88"
        },  
        "device_properties_verify": {
            "lastMaintenance":"25.3.2025 1300", 
            "prop2":"value2",
            "prop77":"value77", 
            "prop88":"value88"
        },  
                  
    }
} 
DeviceCfg_D08 = {
    "name":"device-d08-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {
            "lastMaintenance":"25.3.2025 1300", 
            "prop2":"value2"
        },        
        "credentials_id": "device-d08-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {
        "device_properties": {
            "+prop77":"value77 and 77 and 77", 
            "+prop88":"value88 and 88 and 88"
        },  
        "device_properties_verify": {
            "lastMaintenance":"25.3.2025 1300", 
            "prop2":"value2",
            "prop77":"value77 and 77 and 77", 
            "prop88":"value88 and 88 and 88"
        },  
                  
    }
} 
DeviceCfg_D09 = {
    "name":"device-d09-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {
            "lastMaintenance":"25.3.2025 1300", 
            "prop2":"value2"
        },        
        "credentials_id": "device-d09-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {
        "device_properties": {
            "-prop77":None,
            "-prop88":None
        },  
        "device_properties_verify": {
            "lastMaintenance":"25.3.2025 1300", 
            "prop2":"value2",
        },  
                  
    }
} 
DeviceCfg_D10 = {
    "name":"device-d10-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {},
        "credentials_id": "device-d10-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {
        "uplink_properties": {
            "+decoding-selector":"adeunis", 
            "+prop88":"value88"
        },  
        "uplink_properties_verify": {
            "decoding-selector":"adeunis", 
            "prop88":"value88"
        },  
    }
} 
DeviceCfg_D11 = {
    "name":"device-d11-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "manufacturer":"PSsystec",
        "model":"Smartbox Mini",
        "sub_model":"NB-IoT",
        "iccid": "89374121317959919924",
        "hardware_version":"2024.1 PC r1",
        "country":"DE",
        "description":"Das ist eine Beschreibung.",
        "label":"this is a label" ,
        "device_type":"Pretty Device" ,
        "device_name":"This is the name of the device",
        "firmware_version":"fw 1.0.1",
        "software_version":"sw 2.0.1",
        "os_version":"os 3.0.1",
        "location":"geo:25.245470,51.454009" ,
        "protocols":["LWM2M"] ,
        "uplink_properties": {},
        "device_properties": {},
        "credentials_id": "device-d11-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA==",    
    },
    "modify_props" : {
        "uplink_properties": {
            "+decoding-selector":"adeunis",             
            "+prop88":"value88"
        },  
        "uplink_properties_verify": {
            "decoding-selector":"adeunis", 
            "prop88":"value88"
        },  
    }
} 
DeviceCfg_D13 = {
    "name":"device-d13-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "credentials_id": "device-d13-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA=="        
    },
    "modify_props" : {},
    "sleep_secs": 20
}
DeviceCfg_D14 = {
    "name":"device-d14-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "credentials_id": "device-d14-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA=="        
    },
    "modify_props" : {},
    "sleep_secs": 20
}
DeviceCfg_D15 = {
    "name":"device-d15-%RUNID%",
    "create_props" : {
        "protocol":"LWM2M",
        "credentials_id": "device-d15-%RUNID%",
        "credentials_secret": "OSRBYUJrckhYbVlCNEJvQA=="        
    },
    "modify_props" : {},
    "sleep_secs": 20,
    "inject_data": {"firstName":"Roland", "lastName":"Baldin"},
}
###################################################################
###################################################################
# MARK: Device group test configurations
###################################################################
###################################################################
DeviceGroupCfg_DG01 = {
    "name":"devicegroup-01-%RUNID%", 
    "create_props" : {},
    "modify_props":{}
}
DeviceGroupCfg_DG02 = {
    "name":"devicegroup-02-%RUNID%", 
    "create_props" : {
        "description":"Das ist eine DeviceGroup"
    },
    "modify_props":{}
}
DeviceGroupCfg_DG03 = {
    "name":"devicegroup-03-%RUNID%", 
    "create_props" : {
        "description":"Das ist eine DeviceGroup"
    },
    "modify_props":{
         "description":"Das ist eine geänderte DeviceGroup"
    }
}
DeviceGroupCfg_DG04 = {
    "name":"devicegroup-04-%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"devicegroup-04-app1-%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },    
}
DeviceGroupCfg_DG05_long = {
    "name":"devicegroup-05-%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"devicegroup-05-app1-%RUNID%-123456789012345678901234567890",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },  
    "device_1": DeviceCfg_D02,      
}
DeviceGroupCfg_DG05_short = {
    "name":"dg05%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"dg5ap%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },  
    "device_1": DeviceCfg_D02,      
}
DeviceGroupCfg_DG06_short = {
    "name":"dg06%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"dg6ap%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },  
    "device_1": {
        "name":"d106-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
    "device_2": {
        "name":"d206-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
}
DeviceGroupCfg_DG07_short = {
    "name":"dg07%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"dg7ap1%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },  
    "application_2": {
        "name":"dg7ap2%RUNID%",
        "create_props":{
            "urls": ["https://dev.scs.iot.telekom.com/scs-callback-dummy"],
        }
    },  
    "device_1": {
        "name":"dg7d1-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
}
DeviceGroupCfg_DG08_short = {
    "name":"dg8%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"dg8ap1%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },  
    "application_2": {
        "name":"dg8ap2%RUNID%",
        "create_props":{
            "urls": ["https://dev.scs.iot.telekom.com/scs-callback-dummy"],
        }
    },  
    "device_1": {
        "name":"dg8d1-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
    "device_2": {
        "name":"dg8d2-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
}
DeviceGroupCfg_DG09_short = { 
    "name":"dg9%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "application_1": {
        "name":"dg9ap1%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
        }
    },  
    "application_2": {
        "name":"dg9ap2%RUNID%",
        "create_props":{
            "urls": ["https://dev.scs.iot.telekom.com/scs-callback-dummy"],
        }
    },  
    "device_1": {
        "name":"dg9d1-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
    "device_2": {
        "name":"dg9d2-%RUNID%",
        "create_props" : {
            "protocol":"LWM2M"
        },
        "modify_props" : {}
    },
}

DeviceGroupCfg_DG10_short = DeviceGroupCfg_DG09_short
DeviceGroupCfg_DG11_short = DeviceGroupCfg_DG09_short
DeviceGroupCfg_DG15_short = DeviceGroupCfg_DG09_short
DeviceGroupCfg_DG16_short = DeviceGroupCfg_DG09_short

DeviceGroupCfg_DG17_short = {
    "name":"dg17%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "sleep_secs": 5,  # Sleep seconds before the uplink message is injected
    "correlation_id":"d2ctest_%NOWSECS%",    
    "application_1": {
        "name":"dg17ap%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor?store-body-key=d2ctest_%NOWSECS%"],
        },
        "get_message_urls":["https://api.scs.iot.telekom.com/message-monitor/body/d2ctest_%NOWSECS%"]
    },  
    "device_1": DeviceCfg_D02,      
}
DeviceGroupCfg_DG18_short = {
    "name":"dg18%RUNID%", 
    "create_props" : {},
    "modify_props":{},
    "sleep_secs": 5,  # Sleep seconds before the uplink message is injected
    "correlation_id":"d2ctest_%NOWSECS%",    
    "application_1": {
        "name":"dg18ap1_%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor?store-body-key=d2ctest_%NOWSECS%_1"],
        },
        "get_message_urls":["https://api.scs.iot.telekom.com/message-monitor/body/d2ctest_%NOWSECS%_1"]
    },  
    "application_2": {
        "name":"dg18ap2_%RUNID%",
        "create_props":{
            "urls": ["https://api.scs.iot.telekom.com/message-monitor?store-body-key=d2ctest_%NOWSECS%_2"],
        },
        "get_message_urls":["https://api.scs.iot.telekom.com/message-monitor/body/d2ctest_%NOWSECS%_2"]
    },  
    "device_1": DeviceCfg_D02,      
}

DeviceGroupCfg_DG20 = DeviceGroupCfg_DG01
DeviceGroupCfg_DG21 = DeviceGroupCfg_DG01

# DeviceGroup02Cfg = {
#     "name":"devicegroup02-%RUNID%", 
#     "create_props" : {
#         "labels":{"deviceType":"SDI People Counter"},
#         "device_ids": [],
#         "application_ids": [],
#         "description": "This is a description"        
#     },
#     "application_1": {
#         "name":"devicegroup02-app1-%RUNID%",
#         "create_props":{
#             "urls": ["https://api.scs.iot.telekom.com/message-monitor"],
#             "labels": {"customerKey":"CUST-001"},        
#         }
#     },
#     "application_2":{
#         "name":"devicegroup02-app2-%RUNID%",
#         "create_props":{
#             "urls": ["https://dev.scs.iot.telekom.com/scs-callback-dummy"],
#             "labels": {"customerKey":"CUST-099", "costCenter":"999888"},        
#         }
#     },
#     "device_1": Device01Cfg,
#     "device_2": Device02Cfg
# }